from persistent import Persistent
from zope.interface import implements
from BTrees.OOBTree import OOBTree
from slc.mailrouter.interfaces import IFriendlyNameStorage


class FriendlyNameStorage(Persistent):
    implements(IFriendlyNameStorage)

    def __init__(self):
        self._forward = OOBTree()  # name -> uid
        self._reverse = OOBTree()  # uid -> name

    def add(self, uid, name):
        """ Map name -> uid. """
        store_name = name.lower()
        if store_name in self._forward:
            raise ValueError("%s already mapped" % name)
        if uid in self._reverse:
            raise ValueError("%s already has a friendly name" % uid)
        self._forward[store_name] = uid
        self._reverse[uid] = store_name

    def remove(self, uid):
        """ Remove mapping. This will be called when a folder is deleted,
            therefore we use the uid. """
        marker = object()
        name = self._reverse.get(uid, marker)
        if name is not marker:
            del(self._reverse[uid])
            try:
                del(self._forward[name])
            except KeyError:
                # If it isn't there, good, that is the outcome we wanted,
                # right?
                pass

    def get(self, name, _marker=None):
        """ Look up name, map uid to an object and return it. """
        return self._forward.get(name.lower(), _marker)

    def lookup(self, uid, _marker=None):
        """ Look up uid, return name. """
        return self._reverse.get(uid, _marker)

    def __getitem__(self, key):
        return self._forward.items()[key]

    def __len__(self):
        return len(self._forward)



import unittest

from slc.mailrouter.utils import MailToFolderRouter
from slc.mailrouter.testing import (
    MAILROUTER_INTEGRATION_TESTING,
    load_mail,
    msginfo_privileged,
    msginfo_unprivileged,
    msginfo_unknown,
    msginfo_upper_case,
)
from slc.mailrouter.exceptions import PermissionError


class TestFolderRouter(unittest.TestCase):
    """ Tests the mail folder router """
    layer = MAILROUTER_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.folder = self.portal.get('mailtest')
        self.mailrouter = MailToFolderRouter()

    def send(self, tmpl_file, msginfo):
        msg = load_mail(tmpl_file, msginfo)
        return self.mailrouter(self.portal, msg)

    def test_privileged_plain(self):
        result = self.send('mail_plain.txt', msginfo_privileged)
        self.assertTrue(result, msg="Mail not accepted")
        self.assertTrue('pixel.gif' in self.folder.objectIds(),
                        msg="Attachment not created")

    def test_privileged_html(self):
        result = self.send('mail_html.txt', msginfo_privileged)
        self.assertTrue(result, msg="Mail not accepted")
        self.assertTrue('pink_grad.gif' in self.folder.objectIds(),
                        msg="Attachment not created")

    def test_privileged_mixed(self):
        result = self.send('mail_mixed.txt', msginfo_privileged)
        self.assertTrue(result, msg="Mail not accepted")
        self.assertTrue('pixel.gif' in self.folder.objectIds(),
                        msg="Attachment not created")

    def test_unprivileged(self):
        self.assertRaises(PermissionError,
                          self.send, 'mail_plain.txt', msginfo_unprivileged)

    def test_unknown(self):
        self.assertRaises(PermissionError,
                          self.send, 'mail_plain.txt', msginfo_unknown)

    def test_local_part_case_insensitive(self):
        result = self.send('mail_plain.txt', msginfo_upper_case)
        self.assertTrue(result, msg="Mail not accepted")



from zope.component import queryUtility
import logging

from slc.mailrouter.interfaces import IFriendlyNameStorage

logger = logging.getLogger(__name__)


def convert_storage_entries_to_lower_case(context):
    storage = queryUtility(IFriendlyNameStorage)
    if not storage:
        logger.info('No storage found, nothing done')
        return
    count = 0
    for entry in storage:
        if entry[0] != entry[0].lower():
            storage.remove(entry[1])
            storage.add(entry[1], entry[0].lower())
            count += 1
    logger.info('Converted {0} of {1} storage entries'.format(
        count, len(storage)))



from zope.interface import Interface


class IMailRouterLayer(Interface):
    """Marker Interface used by as BrowserLayer
    """


class IMailImportAdapter(Interface):
    def add(message):
        """ Add message to context. """


class IMappingStorage(Interface):
    """ Stores mappings from names to targets. """

    def add(uid, name):
        """ Map name -> uid. """

    def remove(uid):
        """ Remove mapping. """

    def get(name):
        """ Look up name, return the uid. """

    def lookup(uid):
        """ Look up uid, return name. """

    def __getitem__(key):
        """ Return item corresponding to key. """

    def __len__():
        """ Return the number of items in storage. """


class IFriendlyNameStorage(IMappingStorage):
    """ A place to store the mapping of friendly names to uids, allowing
        a user to email friendlyname@ourdomain instead of uid@ourdomain. """


class IMailRouter(Interface):
    """ Utilities that act as mail routers implement this interface. """

    def __call__(self, site, message):
        """ Calling the utility handles ``message``, return True if it was
            handled, false otherwise. ``site`` is the plone site.
        """

    def priority(self):
        """ Return priority to determine the order in which routers are called.
        """



import re
import email
from copy import copy
from AccessControl.SecurityManagement import newSecurityManager, \
    getSecurityManager
from zope.component import getUtility
from zope.component import queryUtility
from zope.interface import implements
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.interfaces import IFolderish
from plone.registry.interfaces import IRegistry
from plone.uuid.interfaces import IUUID
from plone import api
from slc.mailrouter.interfaces import IFriendlyNameStorage
from slc.mailrouter.interfaces import IMailRouter, IMailImportAdapter
from slc.mailrouter.exceptions import (PermissionError, NotFoundError,
                                       ConfigurationError)
from slc.mailrouter import MessageFactory as _
from logging import getLogger
try:
    from plone.app.async.interfaces import IAsyncService
except ImportError:
    pass

logger = getLogger('slc.mailrouter.utils')

UIDRE = re.compile('^[0-9a-f-]+$')


def get_use_email_as_login():
    try:
        from Products.CMFPlone.interfaces import ISecuritySchema
        version = 5
    except ImportError:
        version = 4

    if version == 5:
        registry = getUtility(IRegistry)
        security_settings = registry.forInterface(
            ISecuritySchema, prefix='plone')
        use_email_as_login = security_settings.use_email_as_login
    else:
        portal_properties = api.portal.get_tool(name='portal_properties')
        site_properties = portal_properties.site_properties
        use_email_as_login = site_properties.getProperty('use_email_as_login')

    return use_email_as_login


def get_user_by_email(email, pm=None):
    if not email:
        return
    user_id = ''
    user = None
    use_email_as_login = get_use_email_as_login()
    pm = api.portal.get_tool(name='portal_membership')

    try:
        # avoid fuzzy searchMembers if we can
        if use_email_as_login:
            user = pm.getMemberById(email).getUser()
        else:
            results = pm.searchMembers('email', email)
            results = [r for r in results
                       if r['email'] == email]
            user_id = results[0]['username']
            user = pm.getMemberById(user_id).getUser()
    except (IndexError, AttributeError):
        return None
    return user


class BaseMailRouter(object):
    def __call__(self, site, msg):
        self.site = site
        self.acl_users = getToolByName(site, 'acl_users')

        sender_from = msg.get('From')
        sender_return_path = msg.get('Return-Path')

        sender_from = email.Utils.parseaddr(sender_from)[1]
        sender_return_path = email.Utils.parseaddr(sender_return_path)[1]
        recipient = email.Utils.parseaddr(msg.get('X-Original-To'))[1]

        if not recipient:
            logger.info('BaseMailRouter could not identify a recipient.' +
                        'No X-Original-To header found.'
                        )
        user = get_user_by_email(sender_return_path)
        if not user:
            user = get_user_by_email(sender_from)
        if not user:
            raise PermissionError(_("No permitted sender address: %s, %s" %
                                  (sender_return_path, sender_from)))

        return self.deliver(msg, user, recipient)


class MailToFolderRouter(BaseMailRouter):
    implements(IMailRouter)

    def deliver(self, msg, user, recipient):
        logger.info('MailToFolderRouter called with message %s from %s to %s' %
                    (msg.get('Message-ID'),
                     user.getProperty('email'),
                     recipient)
                    )
        local_part = recipient.split('@')[0].lower()

        assert len(local_part) <= 50, \
            "local_part must have a reasonable length"

        # Find the right context. Do that by looking up local_part
        # in our local friendly-name storage, and if not found, check
        # if it looks like a uid. Then look up the uid.
        storage = queryUtility(IFriendlyNameStorage)
        uid = storage.get(local_part, None)
        if uid is None and UIDRE.match(local_part) is not None:
            uid = local_part

        if uid is None:
            # local_part is not a uid and is not mapped onto a folder,
            # this is not something we can handle.
            return False

        # Drop privileges to the right user
        self.acl_users = getToolByName(self.site, 'acl_users')
        newSecurityManager(None, user.__of__(self.acl_users))
        context = api.content.get(UID=uid)
        if not context:
            raise NotFoundError(_("Folder not found"))

        if not IFolderish.providedBy(context):
            raise NotFoundError(
                _("Target %s is not a folder" % context.getId())
            )

        result = False

        # Check permissions
        if not getSecurityManager().checkPermission(
                'Add portal content',
                context):
            raise PermissionError(
                _("%s has insufficient privileges on %s" % (
                  user.getProperty('email'),
                  context.getId()
                  ))
            )

        # Defer actual work to an adapter
        result = IMailImportAdapter(context).add(msg)

        return result

    def priority(self):
        return 50


class MailToGroupRouter(BaseMailRouter):
    implements(IMailRouter)

    def _findGroup(self, site, recipient):
        local_part = recipient.split('@')[0]

        assert len(local_part) <= 50, \
            "local_part must have a reasonable length"

        groups_tool = getToolByName(site, 'portal_groups')
        group = groups_tool.getGroupById(local_part)
        if not group:
            pat = re.compile('^%s$' % local_part, re.I)  # ignore case
            candidates = filter(lambda g: pat.match(g),
                                groups_tool.getGroupIds())
            if len(candidates) > 1:
                raise ConfigurationError(
                    'Group name "%s" is not unique' % local_part
                )
        if not group and not candidates:
            return None

        if not group and candidates:
            group = groups_tool.getGroupById(candidates[0])
        return group

    def _sendMailToGroup(self, site, msg, group):
        # get members and send messages
        members = group.getGroupMembers()

        mto = [mmbr.getProperty('email') for mmbr in members]

        logger.info('Sending message with ID %s to recipients %s' %
                    (msg.get('Message-ID'), ', '.join(mto)))
        send_batched(site, msg, mto)

    def deliver(self, msg, user, recipient):
        logger.info('MailToFolderRouter called with message %s from %s to %s' %
                    (msg.get('Message-ID'),
                     user.getProperty('email'),
                     recipient)
                    )

        # Find the group
        group = self._findGroup(self.site, recipient)
        if not group:
            # recipient not a group, we're not handlig this msg
            return False

        logger.info('Resolved message with ID %s for group %s' %
                    (msg.get('Message-ID'), group.getId()))
        self._sendMailToGroup(self.site, msg, group)

        return True

    def priority(self):
        return 30


def send_batched(context, msg, mto):
    mfrom = msg.get('Return-Path')
    msg_out = copy(msg)
    del msg_out['Return-Path']
    del msg_out['X-Original-To']
    for batch in [mto[i:i + 50] for i in range(0, len(mto), 50)]:
        context.MailHost._send(mfrom, batch, msg_out.as_string())
        logger.info("Distributing group mail for '%s' to recipients %s" % (
            msg['X-Original-To'], ', '.join(batch))
        )


# for use in async
def sendMailToGroup(context, msg, groupid):
    # get members and send messages
    acl_users = getToolByName(context, 'acl_users')
    group = acl_users.getGroupById(groupid)
    members = group.getGroupMembers()

    mto = [mmbr.getProperty('email') for mmbr in members]

    send_batched(context, msg, mto)


class AsyncMailToGroupRouter(MailToGroupRouter):
    implements(IMailRouter)

    def _sendMailToGroup(self, site, msg, group):
        async = queryUtility(IAsyncService, default=None, context=self)
        async.queueJob(sendMailToGroup, site, msg, group.id)


def store_name(context, name):
    errors = {}
    storage = queryUtility(IFriendlyNameStorage)
    target = IUUID(context)
    if not name:
        errors.update(
            {'name': _(u'You must provide a friendly name.')})
    elif not re.match(r'^[a-zA-Z0-9_./-]+$', name):
        errors.update(
            {'name': _(u'Forbidden characters in friendly name. '
                       'Allowed characters: a-zA-Z0-9_./-')})
    if not errors:
        existing = storage.get(name)
        if existing and not existing == target:
            errors.update(
                {'name': _(u'This name is already in use.')})
        else:
            storage.remove(target)  # No effect if target isn't mapped
            storage.add(target, name)

    return errors



from datetime import datetime
from zope.component import adapts, queryUtility
from plone.i18n.normalizer.interfaces import IFileNameNormalizer
from plone.rfc822.interfaces import IPrimaryFieldInfo
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.interfaces import IFolderish


class FolderAdapter(object):
    adapts(IFolderish)

    def __init__(self, context):
        self.context = context
        self.normalizer = queryUtility(IFileNameNormalizer)
        self.registry = getToolByName(context, 'content_type_registry')

    def add(self, message):
        # Extract the various parts
        for part in message.walk():
            if part.is_multipart():
                # Ignore the multipart container, we will eventually walk
                # through all of its contents.
                continue

            file_name = part.get_filename()
            if file_name is None:
                # This is probably inline, ignore it
                continue

            content_type = part.get_content_type()
            payload = part.get_payload(decode=1)
            type_name = self.registry.findTypeName(
                file_name,
                content_type,
                payload)

            # Normalise file_name into a safe id
            name = self.normalizer.normalize(file_name)

            # Check that the name is safe to use, else add a date to it
            if name in self.context.objectIds():
                parts = name.split('.')
                parts[0] += datetime.now().strftime('-%Y-%m-%d-%H-%M-%S')
                name = '.'.join(parts)

            self.context.invokeFactory(type_name, name)
            obj = self.context._getOb(name)
            info = IPrimaryFieldInfo(obj, None)
            if info is not None:
                info.field.set(obj, payload)
            else:
                if hasattr(obj, 'getPrimaryField'):
                    obj.getPrimaryField().getMutator(obj)(payload)
                else:
                    raise AttributeError('Could not get primary field')
            obj.setTitle(file_name)
            obj.reindexObject(idxs='Title')
            return True



import email
import sys
from tempfile import NamedTemporaryFile
import traceback
from StringIO import StringIO

from Acquisition import aq_inner
from zope.component import queryUtility, getAllUtilitiesRegisteredFor
from zope.interface import alsoProvides
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFPlone.PloneBatch import Batch
from Products.statusmessages.interfaces import IStatusMessage
from Products.CMFCore.interfaces import IFolderish
from Products.CMFCore.utils import _checkPermission
from Products.CMFCore.permissions import AddPortalContent
from plone.uuid.interfaces import IUUID
from slc.mailrouter.interfaces import IFriendlyNameStorage
from slc.mailrouter.interfaces import IMailRouter
from slc.mailrouter.exceptions import PermanentError, TemporaryError
from slc.mailrouter import MessageFactory as _
from slc.mailrouter.utils import store_name

from logging import getLogger

logger = getLogger('slc.mailrouter.browser')


def get_exception_message(e):
    return '%s: %s' % (e.__class__.__name__, str(e))


def get_exception_log_entry(e):
    exc_type, val, tb = sys.exc_info()
    tb_str = StringIO()
    traceback.print_tb(tb, file=tb_str)
    return '{0}: {1}\nTraceback:\n{2}{0}: {1}'.format(
        e.__class__.__name__, str(e), tb_str.getvalue())


class InjectionView(BrowserView):
    def __call__(self):
        self.request.stdin.seek(0)
        msg = email.message_from_file(self.request.stdin)

        # Get all registered mail routers. Sort by priority, then
        # call them until it is handled
        routers = getAllUtilitiesRegisteredFor(IMailRouter)
        routers.sort(lambda x, y: cmp(x.priority(), y.priority()))
        for router in routers:
            try:
                if router(aq_inner(self.context), msg):
                    try:
                        from plone.protect.interfaces import \
                            IDisableCSRFProtection
                    except ImportError:
                        pass
                    else:
                        alsoProvides(self.request, IDisableCSRFProtection)
                    return 'OK: Message accepted'
            except (PermanentError, TemporaryError), e:
                errmsg = get_exception_message(e)
                self.request.response.setStatus(e.status)
                logmsg = get_exception_log_entry(e)
                logger.warn(logmsg)
                self.dump_mail()
                return 'Fail: %s' % errmsg
            except Exception, e:
                # raise
                errmsg = get_exception_message(e)
                self.request.response.setStatus(500, reason=errmsg)
                logmsg = get_exception_log_entry(e)
                logger.error(logmsg)
                self.dump_mail()
                return 'Fail: %s' % errmsg

        self.request.response.setStatus(404)
        logger.warn('FAIL: Recipient address X-Original-To: %s not found' %
                    (msg.get('X-Original-To')))
        return 'FAIL: Recipient address X-Original-To: %s not found' % \
            (msg.get('X-Original-To'))

    def dump_mail(self):
        try:
            tmpfile = NamedTemporaryFile(
                prefix='mailrouter-dump', delete=False)
            self.request.stdin.seek(0)
            tmpfile.write(self.request.stdin.read())
            logger.info('Dumped mail to %s' % tmpfile.name)
            tmpfile.close()
        except Exception as e:
            logger.warn('Error while dumping mail: %s' %
                        get_exception_log_entry(e))


class FriendlyNameStorageView(BrowserView):
    def update(self):
        """ Called from the template, it deletes any mappings
            specified on the request. """
        remove = self.request.get('remove', ())
        storages = [queryUtility(IFriendlyNameStorage), ]
        for item in remove:
            for storage in storages:
                storage.remove(item)

    def folder_mappings(self):
        storage = queryUtility(IFriendlyNameStorage)
        b_size = int(self.request.get('b_size', 50))
        b_start = int(self.request.get('b_start', 0))
        return Batch(storage, b_size, b_start)


class FriendlyNameAddView(BrowserView):
    addtemplate = ViewPageTemplateFile("add.pt")

    def __call__(self):
        errors = {}
        if self.request.get('form.submitted', None) is not None:
            errors = store_name(self.context, self.request.get('name', ''))

        self.request['errors'] = errors
        if not errors and 'redirect' in self.request:
            IStatusMessage(self.request).add(_(u"Mail route enabled."))
            return self.request.RESPONSE.redirect(self.request['redirect'])
        return self.addtemplate()

    def friendlyname(self):
        storage = queryUtility(IFriendlyNameStorage)
        return storage.lookup(IUUID(self.context))

    def displayMailTab(self):
        # Cannot mail into the portal root
        if self.context.restrictedTraverse(
                '@@plone_context_state').is_portal_root():
            return False
        return IFolderish.providedBy(self.context) and \
            _checkPermission(AddPortalContent, self.context)



from Products.CMFCore.utils import getToolByName
from plone import api
from plone.app.testing import applyProfile
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing.helpers import login, logout
import email
import os
from plone.uuid.interfaces import IUUID
from zope.component import queryUtility
from zope.component.hooks import getSiteManager

from slc.mailrouter.interfaces import IFriendlyNameStorage

PRIVILEGED_USER = "allowed@mailrouter.com"
UNPRIVILEGED_USER = "forbidden@mailrouter.com"
UNKNOWN_USER = "unknown@mailrouter.com"
PASSWORD = "password"


msginfo_privileged = {'FROM': PRIVILEGED_USER,
                      'FOLDERADDR': 'mailtest@mailrouter.com',
                      }

msginfo_unprivileged = {'FROM': UNPRIVILEGED_USER,
                        'FOLDERADDR': 'mailtest@mailrouter.com',
                        }

msginfo_unknown = {'FROM': UNKNOWN_USER,
                   'FOLDERADDR': 'mailtest@mailrouter.com',
                   }

msginfo_upper_case = {'FROM': PRIVILEGED_USER,
                      'FOLDERADDR': 'MAILTEST@mailrouter.com',
                      }


def load_mail(tmpl_file, msginfo):
    fd = open_mailfile(tmpl_file)
    tmpl = fd.read()
    mail = tmpl % msginfo
    msg = email.message_from_string(mail)
    return msg


def open_mailfile(tmpl_file):
    testfolder = os.path.join(os.path.split(__file__)[0], 'tests')
    path = os.path.join(testfolder, tmpl_file)
    fd = open(path)
    return fd


class MailRouterLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import slc.mailrouter
        self.loadZCML(
            'configure.zcml',
            package=slc.mailrouter,
        )
        try:
            import plone.app.contenttypes
        except ImportError:
            self.have_pacontenttypes = False
        else:
            self.have_pacontenttypes = True
            self.loadZCML(
                'configure.zcml',
                package=plone.app.contenttypes,
            )
        gsm = getSiteManager()
        from Products.CMFCore.interfaces import IFolderish
        from slc.mailrouter.interfaces import IMailImportAdapter
        from slc.mailrouter.adapters import FolderAdapter
        gsm.registerAdapter(
            FolderAdapter,
            required=(IFolderish, ),
            provided=IMailImportAdapter)

    def _createUsers(self, portal):
        acl_users = getToolByName(portal, 'acl_users')
        acl_users.userFolderAddUser(
            PRIVILEGED_USER,
            PASSWORD,
            ['Contributor', ],
            [],
        )
        acl_users.userFolderAddUser(
            UNPRIVILEGED_USER,
            PASSWORD,
            [],
            [],
        )
        pm = api.portal.get_tool(name='portal_membership')
        pm.getMemberById(PRIVILEGED_USER).setMemberProperties(
            {'email': PRIVILEGED_USER})
        pm.getMemberById(UNPRIVILEGED_USER).setMemberProperties(
            {'email': UNPRIVILEGED_USER})

    def _createContent(self, portal):
        login(portal, PRIVILEGED_USER)
        portal.invokeFactory('Folder', 'mailtest')
        storage = queryUtility(IFriendlyNameStorage)
        target = IUUID(portal.get('mailtest'))
        storage.add(target, 'mailtest')
        logout

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'slc.mailrouter:default')
        if self.have_pacontenttypes:
            applyProfile(portal, 'plone.app.contenttypes:default')
        self._createUsers(portal)
        self._createContent(portal)

MAILROUTER_FIXTURE = MailRouterLayer()
MAILROUTER_INTEGRATION_TESTING = IntegrationTesting(
    bases=(MAILROUTER_FIXTURE, ), name="SlcMailrouter:Integration")



class PermanentError(Exception):
    """ Base class for all permanent errors. """
    status = 400


class TemporaryError(Exception):
    """ Base class for temporary errors. """
    status = 500


class PermissionError(PermanentError):
    """ Raise this if not enough permission to deliver email. """
    status = 403


class NotFoundError(PermanentError):
    """ Raise this if target cannot be found. """
    status = 404


class ConfigurationError(TemporaryError):
    """ Raise this for configuration errors. """



from subprocess import CalledProcessError
from subprocess import check_call

import os
import shlex
import sys


python_path = sys.executable
bin_dir = os.path.dirname(python_path)
pip_path = os.path.join(bin_dir, "pip")
out_path = "{0}/bin".format(os.getcwd())
bootstrap_clean = "{0} uninstall -y zc.buildout".format(pip_path)
bootstrap = (
    '{0} install -r requirements.txt --install-option="--install-scripts={1}"'
).format(pip_path, out_path)

if not os.path.exists(pip_path):
    print ("pip is not installed in your virtualenv. Reinstall your "
           "virtualenv without using the --no-setuptools or --no-pip options.")
    sys.exit(1)
try:
    print "Cleaning up from previous bootstrap: {0}".format(bootstrap_clean)
    check_call(bootstrap_clean.split(" "))
except CalledProcessError:
    print "Ready for bootstrap"
try:
    print "Running bootstrap command: {0}".format(bootstrap)
    check_call(shlex.split(bootstrap))
    print "Bootstrap complete"
except CalledProcessError:
    print "Please try to bootstrap manually using: {0}".format(bootstrap)



from setuptools import setup, find_packages

version = '2.0.3'

setup(
    name='slc.mailrouter',
    version=version,
    description="Framework for handling email in zope",
    long_description=open("README.txt").read(),
    # Get more strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Programming Language :: Python",
        "Framework :: Plone",
        ],
    keywords='smtp2zope email mailin zope',
    author='Syslab.com GmbH',
    author_email='info@syslab.com',
    url='https://github.com/syslabcom/slc.mailrouter',
    license='GPL',
    packages=find_packages('src', exclude=['ez_setup']),
    package_dir = {'' : 'src'},
    namespace_packages=['slc'],
    include_package_data=True,
    package_data={'slc.mailrouter': ['sql/*']},
    zip_safe=False,
    extras_require={
        'test': [
            'mock',
            'plone.app.testing',
        ],
    },
    install_requires=[
        'setuptools',
        'plone.api',
        'Products.CMFPlone',
    ],
    entry_points="""
        [z3c.autoinclude.plugin]
        target = plone
    """,
    setup_requires=[],
    paster_plugins = [],
    )



import unittest
from mock import Mock
from Testing.makerequest import makerequest
from zope.component.hooks import getSiteManager
from zope.publisher.http import HTTPResponse
from StringIO import StringIO
from logging import getLogger, StreamHandler, INFO
from Products.CMFPlone.utils import safe_unicode

from slc.mailrouter.testing import (
    MAILROUTER_INTEGRATION_TESTING,
    open_mailfile,
)
from slc.mailrouter.interfaces import IMailRouter
from slc.mailrouter.exceptions import PermanentError


class TestInjection(unittest.TestCase):
    """ Tests the mail router injection view """
    layer = MAILROUTER_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.log = StringIO()
        browser_logger = getLogger('slc.mailrouter.browser')
        browser_logger.setLevel(INFO)
        browser_logger.addHandler(StreamHandler(self.log))

    def register_error_mail_router(self, error_class):
        sm = getSiteManager()
        MockMailRouter = Mock(
            return_value=False,
            side_effect=error_class('An error happened!'))
        MockMailRouter.getPhysicalPath = Mock(
            return_value='/plone/mockmailrouter')
        sm.registerUtility(
            name='mockmail',
            provided=IMailRouter,
            component=MockMailRouter)

    def do_inject(self, encoding=None):
        inject = self.portal.restrictedTraverse('@@mailrouter-inject')
        inject.request = makerequest(self.portal)
        inject.request.response = HTTPResponse()
        mailfile = open_mailfile('mail_plain.txt')
        mail = mailfile.read()
        mailfile.close()
        if encoding is not None:
            mail = safe_unicode(mail).encode(encoding)
        inject.request.stdin = StringIO(mail)
        return inject()

    def test_permanent_error(self):
        self.register_error_mail_router(PermanentError)
        result = self.do_inject()

        self.log.flush()
        self.assertTrue(result.startswith("Fail:"),
                        msg="Inject should have failed but didn't")
        self.assertTrue('PermanentError' in result)
        self.assertTrue('Dumped mail' in self.log.getvalue())

    def test_key_error(self):
        self.register_error_mail_router(KeyError)
        result = self.do_inject()

        self.log.flush()
        self.assertTrue(result.startswith("Fail:"),
                        msg="Inject should have failed but didn't")
        self.assertTrue('KeyError' in result)
        self.assertTrue('Dumped mail' in self.log.getvalue())

    def test_not_accepted(self):
        result = self.do_inject()
        self.assertTrue(result.startswith("FAIL:"),
                        msg="Inject should have failed but didn't")
        self.assertTrue('Dumped mail' not in self.log.getvalue())
        self.assertTrue('not found' in self.log.getvalue())

    def test_iso_8859_11_encoding_permanent_error(self):
        self.register_error_mail_router(PermanentError)
        result = self.do_inject(encoding='iso-8859-11')

        self.log.flush()
        self.assertTrue(result.startswith("Fail:"),
                        msg="Inject should have failed but didn't")
        self.assertTrue('PermanentError' in result)
        self.assertTrue('Dumped mail' in self.log.getvalue())
